@@ -75,6 +75,10 @@
 #include <stdlib.h>
 #endif
 
+#ifdef HAVE_SYS_SOCKET_H
+#include <sys/socket.h>
+#endif
+
 #ifdef HAVE_STRING_H
 #include <string.h>
 #endif
@@ -140,6 +144,13 @@
 #include <sys/select.h>
 #endif
 
+#ifdef HAVE_SYS_MODE_H
+/* apparently AIX needs this for S_ISLNK */
+#ifndef S_ISLNK
+#include <sys/mode.h>
+#endif
+#endif
+
 #ifdef HAVE_FNMATCH
 #include <fnmatch.h>
 #else
